%# BEGIN BPS TAGGED BLOCK {{{
%# 
%# COPYRIGHT:
%#  
%# This software is Copyright (c) 1996-2005 Best Practical Solutions, LLC 
%#                                          <jesse@bestpractical.com>
%# 
%# (Except where explicitly superseded by other copyright notices)
%# 
%# 
%# LICENSE:
%# 
%# This work is made available to you under the terms of Version 2 of
%# the GNU General Public License. A copy of that license should have
%# been provided with this software, but in any event can be snarfed
%# from www.gnu.org.
%# 
%# This work is distributed in the hope that it will be useful, but
%# WITHOUT ANY WARRANTY; without even the implied warranty of
%# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%# General Public License for more details.
%# 
%# You should have received a copy of the GNU General Public License
%# along with this program; if not, write to the Free Software
%# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%# 
%# 
%# CONTRIBUTION SUBMISSION POLICY:
%# 
%# (The following paragraph is not intended to limit the rights granted
%# to you to modify and distribute this software under the terms of
%# the GNU General Public License and is only of importance to you if
%# you choose to contribute your changes and enhancements to the
%# community by submitting them to Best Practical Solutions, LLC.)
%# 
%# By intentionally submitting any modifications, corrections or
%# derivatives to this work, or any other work intended for use with
%# Request Tracker, to Best Practical Solutions, LLC, you confirm that
%# you are the copyright holder for those contributions and you grant
%# Best Practical Solutions,  LLC a nonexclusive, worldwide, irrevocable,
%# royalty-free, perpetual, license to use, copy, create derivative
%# works based on those contributions, and sublicense and distribute
%# those contributions and any derivatives thereof.
%# 
%# END BPS TAGGED BLOCK }}}
<& /Elements/Header, Title => $title &>
<& /Admin/Elements/ToolTabs,
    current_toptab => 'Admin',
    current_tab => 'Admin/Tools/RightsMatrix/',
    Title => $title &>

<ul>
%    foreach my $key (sort keys %$tabs) {
<li><font size="+2"><a href="<%$RT::WebPath%>/<%$tabs->{$key}->{'path'}|n %>"><%$tabs->{$key} ->{'title'}%></a></font><br>
<%$tabs->{$key}->{description}%>
</li>
%}
</ul>

<%INIT>
my $tabs = {
    A => {
        title       => loc('Queue Rights'),
        path        => 'Admin/Tools/RightsMatrix/QueueRightsMatrix.html',
        description => loc('Manage queue rights'),
    },
    B => {
        title => loc('Ticket Custom Field Rights'),
        path        => 'Admin/Tools/RightsMatrix/QueueCFRightsMatrix.html',
        description => loc('Manage ticket custom field rights'),
    },
    C => {
        title       => loc('Asset Type Rights'),
        path        => 'Admin/Tools/RightsMatrix/AssetRightsMatrix.html',
        description => loc('Manage asset type rights'),
    },
    D => {
        'title'     => loc('Asset Custom Field Rights'),
        path        => 'Admin/Tools/RightsMatrix/AssetCFRightsMatrix.html',
        description => loc('Manage asset custom field rights'),
    },
};

delete $tabs->{C} unless defined $RTx::AssetTracker::VERSION;
delete $tabs->{D} unless defined $RTx::AssetTracker::VERSION;

$m->comp('/Elements/Callback', tabs => $tabs, %ARGS);


</%INIT>
<%ARGS>
$title => loc('Rights Matrix');
</%ARGS>
